/* SOLAR - Effective Soundness-Guided Reflection Analysis
 *
 * Copyright (C) 2015  Yue Li, Tian Tan, Jingling Xue
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * This is a auxiliary class which is used to introduce the Unknown type into 
 * the existing type system. Not introducing the Unknown type directly by rules
 * or generator is to avoid the AssignCompatible predicate from Unknown to
 * java.lang.Object which may cause performance bug.
 * 
 * In addition, this class can help Solar soundly model the pointer-affecting
 * methods which is directly invoked on the unknown objects during LHM (Lazy
 * Heap Modelling). The heaps (the unknown objects) are lazily modelled, but they
 * may affect the pointer analysis in the paths from newInstance() to the LHM points.
 * However, as the unknown object is with declared type java.lang.Object, it has no
 * fields and thus we only need to model the side effects introduced by the calls to the 
 * methods declared in java.lang.Object. 
 * 
 * Note that it is very rare in practice (i.e., calling methods of java.lang.Object
 * before the LHM points), so we assume there is no such usage patterns in our SAS'15
 * paper. However, for achieving a more comprehensive handling for LHM (considering the
 * theoretical possibility), we handle this issue in our implementation. 
 *
 * Note that all the results in the evaluation of SAS'15 are achieved under this 
 * implementation (although we consider it as an assumption in our paper to save space). 
 *
 * There are only two methods in java.lang.Object need to modelled regarding 
 * pointer analysis:
 * (1) toString()
 * (2) getClass()
 *
 * As for clone() method, it cannot be invoked on a variable with java.lang.Object
 * as declared type directly.
 * 
 * This class overrides toString() by returning an java.lang.String object so that
 * its side-effect can be soundly handled by pointer analysis rules for grant.
 * The rules for handling getClass() reside in logic/library/reflection.logic.
 * Actually, the rules return an unknown Class object to model getClass() soundly.
 *
 * Last comment at 10-09-2015
 * 
 */
public class _Unknown_ {
    public String toString() { return new String(); }
}
